package de.meinefirma.meinprojekt.dao;

import java.util.List;
import javax.naming.*;
import javax.persistence.*;
import javax.transaction.UserTransaction;

/** Super-Klasse fuer alle DAOs */
public abstract class AbstractDao
{
   protected final EntityManager           em;
   protected final EntityOrUserTransaction tx;

   // Konstruktor fuer Betrieb im App-Server:
   public AbstractDao() throws NamingException
   {
      // Container-managed EntityManager, JTA-verwaltete UserTransaction (Type "JTA"):
      em = (EntityManager) (new InitialContext()).lookup( "java:comp/env/persistence/em" );
      tx = new EntityOrUserTransaction( (UserTransaction) (new InitialContext()).lookup( "java:comp/UserTransaction" ) );
   }

   // Konstruktor fuer JUnit-Test:
   public AbstractDao( EntityManager em, UserTransaction tx )
   {
      if( em == null ) {
         // Application-managed EntityManager, Application-managed EntityTransaction (Type "RESOURCE_LOCAL"):
         this.em = Persistence.createEntityManagerFactory( "MeineJpaPU_TEST" ).createEntityManager();
         this.tx = new EntityOrUserTransaction( this.em.getTransaction() );
      } else if( tx == null ) {
         // Application-managed EntityManager, Application-managed EntityTransaction (Type "RESOURCE_LOCAL"):
         this.em = em;
         this.tx = new EntityOrUserTransaction( this.em.getTransaction() );
      } else {
         // Container-managed EntityManager, JTA-verwaltete UserTransaction (Type "JTA"):
         this.em = em;
         this.tx = new EntityOrUserTransaction( tx );
      }
   }

   // Erstelle neuen Eintrag:
   public <T> void createEntity( T entity )
   {
      try {
         tx.begin();
         em.persist( entity );
         tx.commit();
      } catch( Exception ex ) {
         try { tx.rollback(); } catch( Exception e ) {/*ok*/}
         throw new RuntimeException( ex );
      }
   }

   // Lies bestimmte Eintraege zu einer Entity-Klasse:
   @SuppressWarnings("unchecked")
   public <T> List<T> queryEntities( String namedQuery, String paramName, String paramValue )
   {
      Query query = em.createNamedQuery( namedQuery );
      query.setParameter( paramName, paramValue );
      return query.getResultList();
   }

   // Lies alle Eintraege zu einer Entity-Klasse:
   @SuppressWarnings("unchecked")
   public <T> List<T> readAllEntities( Class<T> clss )
   {
      return em.createQuery( "Select d from " + clss.getSimpleName() + " d" ).getResultList();
   }

   // Loesche alle Eintraege zu einer Entity-Klasse:
   public <T> int deleteAllEntities( Class<T> clss )
   {
      try {
         tx.begin();
         em.flush();
         em.clear();
         int r = em.createQuery( "Delete from " + clss.getSimpleName() ).executeUpdate();
         tx.commit();
         return r;
      } catch( Exception ex ) {
         try { tx.rollback(); } catch( Exception e ) {/*ok*/}
         throw new RuntimeException( ex );
      }
   }
}
